/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ex: set tabstop=8 softtabstop=4 shiftwidth=4 expandtab: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla PostScript driver printer list component.
 *
 * The Initial Developer of the Original Code is
 * Kenneth Herron <kherron+mozilla@fmailbox.com>
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsCUPSShim_h___
#define nsCUPSShim_h___

#include "prtypes.h"
#include "psSharedCore.h"


/* Various CUPS data types. We don't #include cups headers to avoid
 * requiring CUPS to be installed on the build host (and to avoid having
 * to test for CUPS in configure).
 */
typedef struct                          /**** Printer Options ****/
{
    char          *name;                  /* Name of option */
    char          *value;                 /* Value of option */
} cups_option_t;

typedef struct               /**** Destination ****/
{
    char          *name,       /* Printer or class name */
                  *instance;   /* Local instance name or NULL */
    int           is_default;  /* Is this printer the default? */
    int           num_options; /* Number of options */
    cups_option_t *options;    /* Options */
} cups_dest_t;

typedef enum                /**** Colorspaces ****/
{
    PPD_CS_CMYK = -4,       /* CMYK colorspace */
    PPD_CS_CMY,             /* CMY colorspace */
    PPD_CS_GRAY = 1,        /* Grayscale colorspace */
    PPD_CS_RGB = 3,         /* RGB colorspace */
    PPD_CS_RGBK,            /* RGBK (K = gray) colorspace */
    PPD_CS_N                /* DeviceN colorspace */
} ppd_cs_t;

typedef enum ppd_ui_e       /**** UI Types ****/
{
    PPD_UI_BOOLEAN,         /* True or False option */
    PPD_UI_PICKONE,         /* Pick one from a list */
    PPD_UI_PICKMANY         /* Pick zero or more from a list */
} ppd_ui_t;

typedef enum ppd_section_e  /**** Order dependency sections ****/
{
    PPD_ORDER_ANY,          /* Option code can be anywhere in the file */
    PPD_ORDER_DOCUMENT,     /* ... must be in the DocumentSetup section */
    PPD_ORDER_EXIT,         /* ... must be sent prior to the document */
    PPD_ORDER_JCL,          /* ... must be sent as a JCL command */
    PPD_ORDER_PAGE,         /* ... must be in the PageSetup section */
    PPD_ORDER_PROLOG        /* ... must be in the Prolog section */
} ppd_section_t;

typedef struct ppd_option_s ppd_option_t;

typedef struct ppd_choice_s     /**** Option choices ****/
{
    char        marked;     /* 0 if not selected, 1 otherwise */
    char        choice[41]; /* Computer-readable option name */
    char        text[81];     /* Human-readable option name */
    char        *code;      /* Code to send for this option */
    ppd_option_t*option;    /* Pointer to parent option structure */
} ppd_choice_t;

struct ppd_option_s         /**** Options ****/
{
    char        conflicted; /* 0 if no conflicts exist, 1 otherwise */
    char        keyword[41];/* Option keyword name ("PageSize", etc.) */
    char        defchoice[41];/* Default option choice */
    char        text[81];   /* Human-readable text */
    ppd_ui_t    ui;         /* Type of UI option */
    ppd_section_t section;  /* Section for command */
    float       order;      /* Order number */
    int         num_choices;/* Number of option choices */
    ppd_choice_t*choices;   /* Option choices */
};

typedef struct          /**** Page Sizes ****/
{
    int         marked;     /* Page size selected? */
    char        name[41];
    /* Media size option */
    float       width,      /* Width of media in points */
                length,     /* Length of media in points */
                left,       /* Left printable margin in points */
                bottom,     /* Bottom printable margin in points */
                right,      /* Right printable margin in points */
                top;        /* Top printable margin in points */
} ppd_size_t;

typedef struct			/**** Files ****/
{
    int         language_level;   /* Language level of device */
    int         color_device;     /* 1 = color device, 0 = grayscale */
    int         variable_sizes;   /* 1 = supports variable sizes, 0 = doesn't */
    int         accurate_screens; /* 1 = supports accurate screens, 0 = not */
    int         contone_only;     /* 1 = continuous tone only, 0 = not */
    int         landscape;         /* -90 or 90 */
    int         model_number;      /* Device-specific model number */
    int         manual_copies;    /* 1 = Copies done manually, 0 = hardware */
    int         throughput;       /* Pages per minute */
    ppd_cs_t    colorspace;       /* Default colorspace */
    char        *patches;         /* Patch commands to be sent to printer */
    int         num_emulations;   /* Number of emulations supported */
    void        *emulations;      /* Emulations and the code to invoke them */
    char        *jcl_begin;       /* Start JCL commands */
    char        *jcl_ps;          /* Enter PostScript interpreter */
    char        *jcl_end;         /* End JCL commands */
    char        *lang_encoding;   /* Language encoding */
    char        *lang_version;    /* Language version (English, Spanish, etc.) */
    char        *modelname;       /* Model name (general) */
    char        *ttrasterizer;    /* Truetype rasterizer */
    char        *manufacturer;    /* Manufacturer name */
    char        *product;         /* Product name (from PS RIP/interpreter) */
    char        *nickname;        /* Nickname (specific) */
    char        *shortnickname;   /* Short version of nickname */
    int         num_groups;       /* Number of UI groups */
    void        *groups;          /* UI groups */
    int         num_sizes;        /* Number of page sizes */
    ppd_size_t  *sizes;           /* Page sizes */
    float       custom_min[2];    /* Minimum variable page size */
    float       custom_max[2];    /* Maximum variable page size */
    float       custom_margins[4];/* Margins around page */
    int         num_consts;       /* Number of UI/Non-UI constraints */
    void        *consts;          /* UI/Non-UI constraints */
    int         num_fonts;        /* Number of pre-loaded fonts */
    char        **fonts;          /* Pre-loaded fonts */
    int         num_profiles;     /* Number of sRGB color profiles */
    void        *profiles;    /* sRGB color profiles */
    int         num_filters;      /* Number of filters */
    char        **filters;        /* Filter strings... */

    /**** New in CUPS 1.1 ****/
    int         flip_duplex;      /* 1 = Flip page for back sides */
    
    /**** New in CUPS 1.1.19 ****/
    char        *protocols;       /* Protocols (BCP, TBCP) string */
    char        *pcfilename;      /* PCFileName string */
    int         num_attrs;        /* Number of attributes */
    int         cur_attr;         /* Current attribute */
    void        **attrs;          /* Attributes */

    /**** New in CUPS 1.2 ****/
    void        *sorted_attrs;    /* Attribute lookup array */
    void        *options;	      /* Option lookup array */
    void        *coptions;	      /* Custom options array */
} ppd_file_t;

typedef cups_dest_t* (PR_CALLBACK *CupsGetDestType)(const char *printer,
                                                    const char *instance,
                                                    int num_dests, 
                                                    cups_dest_t *dests);
typedef int (PR_CALLBACK *CupsGetDestsType)(cups_dest_t **dests);
typedef int (PR_CALLBACK *CupsFreeDestsType)(int         num_dests,
                                             cups_dest_t *dests);
typedef int (PR_CALLBACK *CupsPrintFileType)(const char    *printer,
                                             const char    *filename,
                                             const char    *title,
                                             int           num_options,
                                             cups_option_t *options);
typedef int (PR_CALLBACK *CupsTempFdType)(char *filename,
                                          int   length);
typedef int (PR_CALLBACK *CupsAddOptionType)(const char    *name,
                                             const char    *value,
                                             int           num_options,
                                             cups_option_t **options);
typedef const char* (PR_CALLBACK *CupsGetPPDType) (const char* name);
typedef ppd_file_t* (PR_CALLBACK *PPDOpenFileType) (const char* filename);
typedef void (PR_CALLBACK *PPDCloseType) (ppd_file_t* ppd);
typedef void (PR_CALLBACK *PPDMarkDefaultsType) (ppd_file_t* ppd);
typedef int (PR_CALLBACK *PPDIsMarkedType) (ppd_file_t* ppd,
                                            const char* keyword,
                                            const char* choice);
typedef ppd_option_t* (PR_CALLBACK *PPDFindOption) (ppd_file_t* ppd,
                                                    const char* option);
typedef ppd_choice_t* (PR_CALLBACK *PPDFindChoice) (ppd_option_t* o,
                                                    const char* choice);

struct PRLibrary;

class NS_PSSHARED nsCUPSShim {
    public:
        nsCUPSShim() : mCupsLib(nsnull) { }
        ~nsCUPSShim();

        /**
         * Initialize this object. Attempt to load the CUPS shared
         * library and find function pointers for the supported
         * functions (see below).
         * @return PR_FALSE if the shared library could not be loaded, or if
         *                  any of the functions could not be found.
         *         PR_TRUE  for successful initialization.
         */
        PRBool Init();

        /**
         * @return PR_TRUE  if the object was initialized successfully.
         *         PR_FALSE otherwise.
         */
        PRBool IsInitialized() { return nsnull != mCupsLib; }

        /* Function pointers for supported functions. These are only
         * valid after successful initialization.
         */
        CupsAddOptionType   mCupsAddOption;
        CupsFreeDestsType   mCupsFreeDests;
        CupsGetDestType     mCupsGetDest;
        CupsGetDestsType    mCupsGetDests;
        CupsPrintFileType   mCupsPrintFile;
        CupsTempFdType      mCupsTempFd;
        CupsGetPPDType      mCupsGetPPD;
        PPDOpenFileType     mPpdOpenFile;
        PPDCloseType        mPpdClose;
        PPDMarkDefaultsType mPpdMarkDefaults;
        PPDIsMarkedType     mPpdIsMarked;
        PPDFindOption       mPpdFindOption;
        PPDFindChoice       mPpdFindChoice;
    private:
        PRLibrary *mCupsLib;
};



#endif /* nsCUPSShim_h___ */
